package istia.st.mvc.personnes.web;

import istia.st.mvc.personnes.dao.DaoException;
import istia.st.mvc.personnes.dao.DaoImpl;
import istia.st.mvc.personnes.entites.Personne;
import istia.st.mvc.personnes.service.ServiceImpl;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@SuppressWarnings("serial")
public class Application extends HttpServlet {
	// paramtres d'instance
	private String urlErreurs = null;

	private ArrayList erreursInitialisation = new ArrayList<String>();

	private String[] paramtres = { "urlList", "urlEdit", "urlErreurs" };

	private Map params = new HashMap<String, String>();

	// service
	ServiceImpl service = null;

	// init
	@SuppressWarnings("unchecked")
	public void init() throws ServletException {
		// on rcupre les paramtres d'initialisation de la servlet
		ServletConfig config = getServletConfig();
		// on traite les autres paramtres d'initialisation
		String valeur = null;
		for (int i = 0; i < paramtres.length; i++) {
			// valeur du paramtre
			valeur = config.getInitParameter(paramtres[i]);
			// paramtre prsent ?
			if (valeur == null) {
				// on note l'erreur
				erreursInitialisation.add("Le paramtre [" + paramtres[i]
						+ "] n'a pas t initialis");
			} else {
				// on mmorise la valeur du paramtre
				params.put(paramtres[i], valeur);
			}
		}
		// l'url de la vue [erreurs] a un traitement particulier
		urlErreurs = config.getInitParameter("urlErreurs");
		if (urlErreurs == null)
			throw new ServletException(
					"Le paramtre [urlErreurs] n'a pas t initialis");
		// instanciation de la couche [dao]
		DaoImpl dao = new DaoImpl();
		dao.init();
		// instanciation de la couche [service]
		service = new ServiceImpl();
		service.setDao(dao);
	}

	// GET
	@SuppressWarnings("unchecked")
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {

		// on vrifie comment s'est passe l'initialisation de la servlet
		if (erreursInitialisation.size() != 0) {
			// on passe la main  la page d'erreurs
			request.setAttribute("erreurs", erreursInitialisation);
			getServletContext().getRequestDispatcher(urlErreurs).forward(
					request, response);
			// fin
			return;
		}
		// on rcupre la mthode d'envoi de la requte
		String mthode = request.getMethod().toLowerCase();
		// on rcupre l'action  excuter
		String action = request.getPathInfo();
		// action ?
		if (action == null) {
			action = "/list";
		}
		// excution action
		if (mthode.equals("get") && action.equals("/list")) {
			// liste des personnes
			doListPersonnes(request, response);
			return;
		}
		if (mthode.equals("get") && action.equals("/delete")) {
			// suppression d'une personne
			doDeletePersonne(request, response);
			return;
		}
		if (mthode.equals("get") && action.equals("/edit")) {
			// prsentation formulaire ajout / modification d'une personne
			doEditPersonne(request, response);
			return;
		}
		if (mthode.equals("post") && action.equals("/validate")) {
			// validation formulaire ajout / modification d'une personne
			doValidatePersonne(request, response);
			return;
		}
		// autres cas
		doListPersonnes(request, response);
	}

	// affichage liste des personnes
	private void doListPersonnes(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// le modle de la vue [list]
		request.setAttribute("personnes", service.getAll());
		// affichage de la vue [list]
		getServletContext()
				.getRequestDispatcher((String) params.get("urlList")).forward(
						request, response);
	}

	// modification / ajout d'une personne
	private void doEditPersonne(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// on rcupre l'id de la personne
		int id = Integer.parseInt(request.getParameter("id"));
		// ajout ou modification ?
		Personne personne = null;
		if (id != -1) {
			// modification - on rcupre la personne  modifier
			personne = service.getOne(id);
		} else {
			// ajout - on cre une personne vide
			personne = new Personne();
			personne.setId(-1);
		}
		// on met l'objet [Personne] dans le modle de la vue [edit]
		request.setAttribute("erreurEdit", "");
		request.setAttribute("id", personne.getId());
		request.setAttribute("version", personne.getVersion());
		request.setAttribute("prenom", personne.getPrenom());
		request.setAttribute("nom", personne.getNom());
		Date dateNaissance = personne.getDateNaissance();
		if (dateNaissance != null) {
			request.setAttribute("dateNaissance", new SimpleDateFormat(
					"dd/MM/yyyy").format(dateNaissance));
		} else {
			request.setAttribute("dateNaissance", "");
		}
		request.setAttribute("marie", personne.getMarie());
		request.setAttribute("nbEnfants", personne.getNbEnfants());
		// affichage de la vue [edit]
		getServletContext()
				.getRequestDispatcher((String) params.get("urlEdit")).forward(
						request, response);
	}

	// validation modification / ajout d'une personne
	private void doDeletePersonne(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// on rcupre l'id de la personne
		int id = Integer.parseInt(request.getParameter("id"));
		// on supprime la personne
		service.deleteOne(id);
		// on redirige vers la liste des personnes
		response.sendRedirect("list");
	}

	// validation modification / ajout d'une personne
	public void doValidatePersonne(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// on rcupre les lments posts
		boolean formulaireErron = false;
		boolean erreur;
		// le prnom
		String prenom = request.getParameter("prenom").trim();
		// prnom valide ?
		if (prenom.length() == 0) {
			// on note l'erreur
			request.setAttribute("erreurPrenom", "Le prnom est obligatoire");
			formulaireErron = true;
		}
		// le nom
		String nom = request.getParameter("nom").trim();
		// prnom valide ?
		if (nom.length() == 0) {
			// on note l'erreur
			request.setAttribute("erreurNom", "Le nom est obligatoire");
			formulaireErron = true;
		}
		// la date de naissance
		Date dateNaissance = null;
		try {
			dateNaissance = new SimpleDateFormat("dd/MM/yyyy").parse(request
					.getParameter("dateNaissance").trim());
System.out.println(dateNaissance);
		} catch (ParseException e) {
			// on note l'erreur
			request.setAttribute("erreurDateNaissance", "Date incorrecte");
			formulaireErron = true;
		}
		// tat marital
		boolean marie = Boolean.parseBoolean(request.getParameter("marie")
				.trim());
		// nombre d'enfants
		int nbEnfants = 0;
		erreur = false;
		try {
			nbEnfants = Integer.parseInt(request.getParameter("nbEnfants")
					.trim());
			if (nbEnfants < 0) {
				erreur = true;
			}
		} catch (NumberFormatException ex) {
			// on note l'erreur
			erreur = true;
		}
		// nombre d'enfants erron ?
		if (erreur) {
			// on signale l'erreur
			request.setAttribute("erreurNbEnfants",
					"Nombre d'enfants incorrect");
			formulaireErron = true;
		}
		// id de la personne
		int id = Integer.parseInt(request.getParameter("id"));
		// version
		long version = Long.parseLong(request.getParameter("version"));
		// le formulaire est-il erron ?
		if (formulaireErron) {
			// on raffiche le formulaire avec les messages d'erreurs
			showFormulaire(request, response, "");
			// fini
			return;
		}
		// le formulaire est correct - on enregistre la personne
		Personne personne = new Personne(id, prenom, nom, dateNaissance, marie,
				nbEnfants);
		personne.setVersion(version);
		try {
			// enregistrement
			service.saveOne(personne);
		} catch (DaoException ex) {
			// on raffiche le formulaire avec le message de l'erreur survenue
			showFormulaire(request, response, ex.getMessage());
			// fini
			return;
		}
		// on redirige vers la liste des personnes
		response.sendRedirect("list");
	}

	// affichage formulaire pr-rempli
	private void showFormulaire(HttpServletRequest request,
			HttpServletResponse response, String erreurEdit)
			throws ServletException, IOException {
		// on prpare le modle de la vue [edit]
		request.setAttribute("erreurEdit", erreurEdit);
		request.setAttribute("id", request.getParameter("id"));
		request.setAttribute("version", request.getParameter("version"));
		request.setAttribute("prenom", request.getParameter("prenom").trim());
		request.setAttribute("nom", request.getParameter("nom").trim());
		request.setAttribute("dateNaissance", request.getParameter(
				"dateNaissance").trim());
		request.setAttribute("marie", request.getParameter("marie"));
		request.setAttribute("nbEnfants", request.getParameter("nbEnfants")
				.trim());
		// affichage de la vue [edit]
		getServletContext()
				.getRequestDispatcher((String) params.get("urlEdit")).forward(
						request, response);
	}

	// post
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		// on passe la main au GET
		doGet(request, response);
	}

}
