package istia.st.servlets.personne;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

@SuppressWarnings("serial")
public class ServletPersonne2 extends HttpServlet {
	// paramtres d'instance
	private String urlErreurs = null;

	private ArrayList erreursInitialisation = new ArrayList<String>();

	private String[] paramtres = { "urlFormulaire", "urlReponse",
			"urlControleur", "lienRetourFormulaire" };

	private Map params = new HashMap<String, String>();

	private String urlControleur = null;

	// init
	@SuppressWarnings("unchecked")
	public void init() throws ServletException {
		// on rcupre les paramtres d'initialisation de la servlet
		ServletConfig config = getServletConfig();
		// on traite les autres paramtres d'initialisation
		String valeur = null;
		for (int i = 0; i < paramtres.length; i++) {
			// valeur du paramtre
			valeur = config.getInitParameter(paramtres[i]);
			// paramtre prsent ?
			if (valeur == null) {
				// on note l'erreur
				erreursInitialisation.add("Le paramtre [" + paramtres[i]
						+ "] n'a pas t initialis");
			} else {
				// on mmorise la valeur du paramtre
				params.put(paramtres[i], valeur);
			}
			// l'url de la vue [erreurs] a un traitement particulier
			urlErreurs = config.getInitParameter("urlErreurs");
			if (urlErreurs == null)
				throw new ServletException(
						"Le paramtre [urlErreurs] n'a pas t initialis");
		}
		// url du contrleur
		if (params.get("urlControleur") != null) {
			urlControleur = (String) params.get("urlControleur");
		}
	}

	@SuppressWarnings("unchecked")
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {

		// on vrifie comment s'est passe l'initialisation de la servlet
		if (erreursInitialisation.size() != 0) {
			// on passe la main  la page d'erreurs
			request.setAttribute("erreurs", erreursInitialisation);
			request.setAttribute("lienRetourFormulaire", "");
			getServletContext().getRequestDispatcher(urlErreurs).forward(
					request, response);
			// fin
			return;
		}

		// on rcupre la session de l'utilisateur
		HttpSession session = request.getSession(true);

		// on rcupre la mthode d'envoi de la requte
		String mthode = request.getMethod().toLowerCase();
		// on rcupre l'action  excuter
		String action = request.getParameter("action");
		// action ?
		if (action == null) {
			action = "init";
		}
		// excution action
		if (mthode.equals("get") && action.equals("init")) {
			// dmarrage application
			doInit(request, response, session);
			return;
		}
		if (mthode.equals("post") && action.equals("validationFormulaire")) {
			// validation du formulaire de saisie
			doValidationFormulaire(request, response, session);
			return;
		}
		if (mthode.equals("get") && action.equals("retourFormulaire")) {
			// retour au formulaire de saisie
			doRetourFormulaire(request, response, session);
			return;
		}
		// autres cas
		doInit(request, response, session);
	}

	// affichage formulaire vide
	void doInit(HttpServletRequest request, HttpServletResponse response,
			HttpSession session) throws ServletException, IOException {
		// on envoie le formulairevide
		session.setAttribute("nom", "");
		session.setAttribute("age", "");
		session.setAttribute("urlAction", urlControleur);
		getServletContext().getRequestDispatcher(
				(String) params.get("urlFormulaire"))
				.forward(request, response);
		return;
	}

	// affichage formulaire pr-rempli
void doRetourFormulaire(HttpServletRequest request, HttpServletResponse response,
			HttpSession session) throws ServletException, IOException {
		// on envoie le formulaire pr-rempli
		// nom prsent dans la session ?
		String nom = (String) session.getAttribute("nom");
		if (nom == null)
			session.setAttribute("nom", "");
		// ge prsent dans la session ?
		String age = (String) session.getAttribute("age");
		if (age == null)
			session.setAttribute("age", "");
		session.setAttribute("urlAction", urlControleur);
		// on envoie le formulaire pr-rempli
		getServletContext().getRequestDispatcher(
				(String) params.get("urlFormulaire"))
				.forward(request, response);
		return;
	}
	// validation du formulaire
	void doValidationFormulaire(HttpServletRequest request,
			HttpServletResponse response, HttpSession session)
			throws ServletException, IOException {
		// on rcupre les paramtres
		String nom = request.getParameter("txtNom");
		String age = request.getParameter("txtAge");
		// on les mmorise dans la session
		session.setAttribute("nom", nom);
		session.setAttribute("age", age);
		// vrification des paramtres
		ArrayList<String> erreursAppel = new ArrayList<String>();
		// le nom doit tre non vide
		nom = nom.trim();
		if (nom.equals(""))
			erreursAppel.add("Le champ [nom] n'a pas t rempli");
		// l'ge doit tre un entier >=0
		if (!age.matches("^\\s*\\d+\\s*$"))
			erreursAppel.add("Le champ [age] est erron");
		// des erreurs dans les paramtres ?
		if (erreursAppel.size() != 0) {
			// on envoie la page d'erreurs
			request.setAttribute("erreurs", erreursAppel);
			request.setAttribute("lienRetourFormulaire", (String) params
					.get("lienRetourFormulaire"));
			getServletContext().getRequestDispatcher(urlErreurs).forward(
					request, response);
			return;
		}
		// les paramtres sont corrects - on envoie la page rponse
		request.setAttribute("nom", nom);
		request.setAttribute("age", age);
		request.setAttribute("lienRetourFormulaire", (String) params
				.get("lienRetourFormulaire"));
		getServletContext().getRequestDispatcher(
				(String) params.get("urlReponse")).forward(request, response);
		return;
	}

	// post
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		// on passe la main au GET
		doGet(request, response);
	}
}
